@@ -54,6 +54,8 @@ struct ReloadConfig{ std::optional<std::string> profileName; };
 struct ResetConfig{};
 struct ResetFontSize{};
 struct ScreenshotVT{};
+struct SaveScreenshot{};
+struct CopyScreenshot{};
 struct ScrollDown{};
 struct ScrollMarkDown{};
 struct ScrollMarkUp{};
@@ -77,10 +79,12 @@ struct TraceLeave{};
 struct TraceStep{};
 struct ViNormalMode{};
 struct WriteScreen{ std::string chars; }; // "\033[2J\033[3J"
-// CloseTab
-// FocusNextTab
-// FocusPreviousTab
-// OpenTab
+struct CreateNewTab{};
+struct CloseTab{};
+struct SwitchToTab{ int position; };
+struct SwitchToPreviousTab{};
+struct SwitchToTabLeft{};
+struct SwitchToTabRight{};
 // clang-format on
 
 using Action = std::variant<CancelSelection,
@@ -109,6 +113,8 @@ using Action = std::variant<CancelSelection,
                             ResetConfig,
                             ResetFontSize,
                             ScreenshotVT,
+                            SaveScreenshot,
+                            CopyScreenshot,
                             ScrollDown,
                             ScrollMarkDown,
                             ScrollMarkUp,
@@ -131,7 +137,13 @@ using Action = std::variant<CancelSelection,
                             TraceLeave,
                             TraceStep,
                             ViNormalMode,
-                            WriteScreen>;
+                            WriteScreen,
+                            CreateNewTab,
+                            CloseTab,
+                            SwitchToTab,
+                            SwitchToPreviousTab,
+                            SwitchToTabLeft,
+                            SwitchToTabRight>;
 
 std::optional<Action> fromString(std::string const& name);
 
@@ -195,6 +207,10 @@ namespace documentation
         "Resets font size to what is configured in the config file."
     };
     constexpr inline std::string_view ScreenshotVT { "Takes a screenshot in form of VT escape sequences." };
+    constexpr inline std::string_view SaveScreenshot { "Takes a screenshot and saves it into a file." };
+    constexpr inline std::string_view CopyScreenshot {
+        "takes a screenshot and puts it into the system clipboard"
+    };
     constexpr inline std::string_view ScrollDown { "Scrolls down by the multiplier factor." };
     constexpr inline std::string_view ScrollMarkDown {
         "Scrolls one mark down (if none present, bottom of the screen)"
@@ -241,6 +257,14 @@ namespace documentation
     constexpr inline std::string_view WriteScreen {
         "Writes VT sequence in `chars` member to the screen (bypassing the application)."
     };
+    constexpr inline std::string_view CreateNewTab { "Creates a new tab in the terminal emulator." };
+    constexpr inline std::string_view CloseTab { "Closes current tab." };
+    constexpr inline std::string_view SwitchToTab {
+        "Switch to absolute tab position (starting at number 1)"
+    };
+    constexpr inline std::string_view SwitchToPreviousTab { "Switch to the previously focused tab" };
+    constexpr inline std::string_view SwitchToTabLeft { "Switch to tab to the left" };
+    constexpr inline std::string_view SwitchToTabRight { "Switch to tab to the right" };
 } // namespace documentation
 
 inline auto getDocumentation()
@@ -272,6 +296,8 @@ inline auto getDocumentation()
         std::tuple { Action { ResetConfig {} }, documentation::ResetConfig },
         std::tuple { Action { ResetFontSize {} }, documentation::ResetFontSize },
         std::tuple { Action { ScreenshotVT {} }, documentation::ScreenshotVT },
+        std::tuple { Action { SaveScreenshot {} }, documentation::SaveScreenshot },
+        std::tuple { Action { CopyScreenshot {} }, documentation::CopyScreenshot },
         std::tuple { Action { ScrollDown {} }, documentation::ScrollDown },
         std::tuple { Action { ScrollMarkDown {} }, documentation::ScrollMarkDown },
         std::tuple { Action { ScrollMarkUp {} }, documentation::ScrollMarkUp },
@@ -295,6 +321,12 @@ inline auto getDocumentation()
         std::tuple { Action { TraceStep {} }, documentation::TraceStep },
         std::tuple { Action { ViNormalMode {} }, documentation::ViNormalMode },
         std::tuple { Action { WriteScreen {} }, documentation::WriteScreen },
+        std::tuple { Action { CreateNewTab {} }, documentation::CreateNewTab },
+        std::tuple { Action { CloseTab {} }, documentation::CloseTab },
+        std::tuple { Action { SwitchToTab {} }, documentation::SwitchToTab },
+        std::tuple { Action { SwitchToPreviousTab {} }, documentation::SwitchToPreviousTab },
+        std::tuple { Action { SwitchToTabLeft {} }, documentation::SwitchToTabLeft },
+        std::tuple { Action { SwitchToTabRight {} }, documentation::SwitchToTabRight },
     };
 }
 
@@ -338,6 +370,8 @@ DECLARE_ACTION_FMT(ReloadConfig)
 DECLARE_ACTION_FMT(ResetConfig)
 DECLARE_ACTION_FMT(ResetFontSize)
 DECLARE_ACTION_FMT(ScreenshotVT)
+DECLARE_ACTION_FMT(SaveScreenshot)
+DECLARE_ACTION_FMT(CopyScreenshot)
 DECLARE_ACTION_FMT(ScrollDown)
 DECLARE_ACTION_FMT(ScrollMarkDown)
 DECLARE_ACTION_FMT(ScrollMarkUp)
@@ -361,9 +395,23 @@ DECLARE_ACTION_FMT(TraceLeave)
 DECLARE_ACTION_FMT(TraceStep)
 DECLARE_ACTION_FMT(ViNormalMode)
 DECLARE_ACTION_FMT(WriteScreen)
+DECLARE_ACTION_FMT(CreateNewTab)
+DECLARE_ACTION_FMT(CloseTab)
+DECLARE_ACTION_FMT(SwitchToPreviousTab)
+DECLARE_ACTION_FMT(SwitchToTabLeft)
+DECLARE_ACTION_FMT(SwitchToTabRight)
 // }}}
 #undef DECLARE_ACTION_FMT
 
+template <>
+struct std::formatter<contour::actions::SwitchToTab>: std::formatter<std::string>
+{
+    auto format(contour::actions::SwitchToTab const& value, auto& ctx) const
+    {
+        return formatter<string>::format(std::format("SwitchToTab {{ position: {} }}", value.position), ctx);
+    }
+};
+
 #define HANDLE_ACTION(T)                                                  \
     if (std::holds_alternative<contour::actions::T>(_action))             \
     {                                                                     \
@@ -402,6 +450,8 @@ struct std::formatter<contour::actions::Action>: std::formatter<std::string>
         HANDLE_ACTION(ResetConfig);
         HANDLE_ACTION(ResetFontSize);
         HANDLE_ACTION(ScreenshotVT);
+        HANDLE_ACTION(CopyScreenshot);
+        HANDLE_ACTION(SaveScreenshot);
         HANDLE_ACTION(ScrollDown);
         HANDLE_ACTION(ScrollMarkDown);
         HANDLE_ACTION(ScrollMarkUp);
@@ -424,6 +474,16 @@ struct std::formatter<contour::actions::Action>: std::formatter<std::string>
         HANDLE_ACTION(TraceLeave);
         HANDLE_ACTION(TraceStep);
         HANDLE_ACTION(ViNormalMode);
+        HANDLE_ACTION(CreateNewTab);
+        HANDLE_ACTION(CloseTab);
+        HANDLE_ACTION(SwitchToPreviousTab);
+        HANDLE_ACTION(SwitchToTabLeft);
+        HANDLE_ACTION(SwitchToTabRight);
+        if (std::holds_alternative<contour::actions::SwitchToTab>(_action))
+        {
+            const auto action = std::get<contour::actions::SwitchToTab>(_action);
+            name = std::format("SwitchToTab, position: {}", action.position);
+        }
         if (std::holds_alternative<contour::actions::WriteScreen>(_action))
         {
             const auto writeScreenAction = std::get<contour::actions::WriteScreen>(_action);
