@@ -60,6 +60,8 @@
 #include <unordered_map>
 #include <variant>
 
+#include <reflection-cpp/reflection.hpp>
+
 namespace contour::config
 {
 
@@ -98,11 +100,6 @@ struct InputMappings
 
 namespace helper
 {
-    inline std::string replaceCommentPlaceholder(std::string const& docString)
-    {
-        return std::regex_replace(docString, std::regex { "\\{comment\\}" }, "#");
-    }
-
     inline bool testMatchMode(uint8_t actualModeFlags,
                               vtbackend::MatchModes expected,
                               vtbackend::MatchModes::Flag testFlag)
@@ -162,6 +159,62 @@ struct CursorConfig
     std::chrono::milliseconds cursorBlinkInterval;
 };
 
+struct HistoryConfig
+{
+    vtbackend::MaxHistoryLineCount maxHistoryLineCount { vtbackend::LineCount(1000) };
+    vtbackend::LineCount historyScrollMultiplier { vtbackend::LineCount(3) };
+    bool autoScrollOnUpdate { true };
+};
+
+struct ScrollBarConfig
+{
+    ScrollBarPosition position { ScrollBarPosition::Hidden };
+    bool hideScrollbarInAltScreen { true };
+};
+
+struct MouseConfig
+{
+    bool hideWhileTyping { true };
+};
+
+struct IndicatorConfig
+{
+    std::string left { " {InputMode:Bold,Color=#FFFF00}"
+                       "{Tabs:ActiveColor=#FFFF00,Left= │ }"
+                       "{SearchPrompt:Left= │ }"
+                       "{TraceMode:Bold,Color=#FFFF00,Left= │ }"
+                       "{ProtectedMode:Bold,Left= │ }" };
+    std::string middle { "{Title:Left= « ,Right= » }" };
+    std::string right { "{HistoryLineCount:Faint,Color=#c0c0c0} │ {Clock:Bold}" };
+};
+
+struct StatusLineConfig
+{
+    vtbackend::StatusDisplayType initialType { vtbackend::StatusDisplayType::Indicator };
+    vtbackend::StatusDisplayPosition position { vtbackend::StatusDisplayPosition::Bottom };
+    bool syncWindowTitleWithHostWritableStatusDisplay { false };
+    IndicatorConfig indicator;
+};
+
+struct BackgroundConfig
+{
+    vtbackend::Opacity opacity { vtbackend::Opacity(0xFF) };
+    bool blur { false };
+};
+
+struct HyperlinkDecorationConfig
+{
+    vtrasterizer::Decorator normal { vtrasterizer::Decorator::DottedUnderline };
+    vtrasterizer::Decorator hover { vtrasterizer::Decorator::Underline };
+};
+
+struct PermissionsConfig
+{
+    Permission captureBuffer { Permission::Ask };
+    Permission changeFont { Permission::Ask };
+    Permission displayHostWritableStatusLine { Permission::Ask };
+};
+
 struct InputModeConfig
 {
     CursorConfig cursor;
@@ -190,6 +243,21 @@ enum class RenderingBackend : uint8_t
     OpenGL,
 };
 
+struct RendererConfig
+{
+    RenderingBackend renderingBackend { RenderingBackend::Default };
+    crispy::lru_capacity textureAtlasTileCount { 4000u };
+    crispy::strong_hashtable_size textureAtlasHashtableSlots { 4096u };
+    bool textureAtlasDirectMapping { false };
+};
+
+struct ImagesConfig
+{
+    bool sixelScrolling { true };
+    vtbackend::ImageSize maxImageSize { vtpty::Width { 0 }, vtpty::Height { 0 } };
+    int maxImageColorRegisters { 4096 };
+};
+
 struct HorizontalMarginTag
 {
 };
@@ -206,13 +274,26 @@ struct WindowMargins
     VerticalMargin vertical { 0 };
 };
 
-template <typename T, documentation::StringLiteral doc>
+constexpr WindowMargins operator*(WindowMargins const& margin, double factor) noexcept
+{
+    return WindowMargins {
+        .horizontal = HorizontalMargin { static_cast<unsigned>(*margin.horizontal * factor) },
+    };
+}
+
+template <typename... T>
 struct ConfigEntry
+{
+};
+
+template <typename T, documentation::StringLiteral configDoc, documentation::StringLiteral webDoc>
+struct ConfigEntry<T, documentation::DocumentationEntry<configDoc, webDoc>>
 {
     using value_type = T;
 
-    std::string documentation = doc.value;
+    std::string documentation = configDoc.value;
     constexpr ConfigEntry(): _value {} {}
+
     constexpr explicit ConfigEntry(T in): _value { std::move(in) } {}
 
     template <typename F>
@@ -315,60 +396,31 @@ struct TerminalProfile
         .env = {},
     } };
     ConfigEntry<vtpty::SshHostConfig, documentation::SshHostConfig> ssh {};
-    ConfigEntry<bool, documentation::Maximized> maximized { false };
-    ConfigEntry<bool, documentation::Fullscreen> fullscreen { false };
+    ConfigEntry<bool, documentation::EscapeSandbox> escapeSandbox { true };
+    ConfigEntry<vtbackend::LineOffset, documentation::CopyLastMarkRangeOffset> copyLastMarkRangeOffset { 0 };
     ConfigEntry<bool, documentation::ShowTitleBar> showTitleBar { true };
     ConfigEntry<bool, documentation::ShowIndicatorOnResize> sizeIndicatorOnResize { true };
-    ConfigEntry<bool, documentation::MouseHideWhileTyping> mouseHideWhileTyping { true };
+    ConfigEntry<bool, documentation::Fullscreen> fullscreen { false };
+    ConfigEntry<bool, documentation::Maximized> maximized { false };
     ConfigEntry<bool, documentation::SeachModeSwitch> searchModeSwitch { true };
-    ConfigEntry<vtbackend::LineOffset, documentation::CopyLastMarkRangeOffset> copyLastMarkRangeOffset { 0 };
-    ConfigEntry<std::string, documentation::WMClass> wmClass { CONTOUR_APP_ID };
-    ConfigEntry<WindowMargins, documentation::Margins> margins { { HorizontalMargin { 0u },
-                                                                   VerticalMargin { 0u } } };
+    ConfigEntry<bool, documentation::InsertAfterYank> insertAfterYank { false };
+    ConfigEntry<Bell, documentation::Bell> bell { { .sound = "default", .alert = true, .volume = 1.0f } };
+    ConfigEntry<vtbackend::VTType, documentation::TerminalId> terminalId { vtbackend::VTType::VT525 };
+    ConfigEntry<std::map<vtbackend::DECMode, bool>, documentation::FrozenDecMode> frozenModes {};
+    ConfigEntry<std::chrono::milliseconds, documentation::SmoothLineScrolling> smoothLineScrolling { 100 };
     ConfigEntry<vtbackend::PageSize, documentation::TerminalSize> terminalSize { {
         vtbackend::LineCount(25),
         vtbackend::ColumnCount(80),
     } };
-    ConfigEntry<vtbackend::VTType, documentation::TerminalId> terminalId { vtbackend::VTType::VT525 };
-    ConfigEntry<vtbackend::MaxHistoryLineCount, documentation::MaxHistoryLineCount> maxHistoryLineCount {
-        vtbackend::LineCount(1000)
-    };
-    ConfigEntry<vtbackend::LineCount, documentation::HistoryScrollMultiplier> historyScrollMultiplier {
-        vtbackend::LineCount(3)
-    };
-    ConfigEntry<ScrollBarPosition, documentation::ScrollbarPosition> scrollbarPosition {
-        ScrollBarPosition::Hidden
-    };
-    ConfigEntry<vtbackend::StatusDisplayPosition, documentation::StatusDisplayPosition>
-        statusDisplayPosition { vtbackend::StatusDisplayPosition::Bottom };
-    ConfigEntry<std::string, documentation::IndicatorStatusLineLeft> indicatorStatusLineLeft {
-        " {InputMode:Bold,Color=#FFFF00}"
-        "{SearchPrompt:Left= │ }"
-        "{TraceMode:Bold,Color=#FFFF00,Left= │ }"
-        "{ProtectedMode:Bold,Left= │ }"
-    };
-    ConfigEntry<std::string, documentation::IndicatorStatusLineMiddle> indicatorStatusLineMiddle {
-        "{Title:Left= « ,Right= » }"
-    };
-    ConfigEntry<std::string, documentation::IndicatorStatusLineRight> indicatorStatusLineRight {
-        "{HistoryLineCount:Faint,Color=#c0c0c0} │ {Clock:Bold}"
-    };
-    ConfigEntry<bool, documentation::SyncWindowTitleWithHostWritableStatusDisplay>
-        syncWindowTitleWithHostWritableStatusDisplay { false };
-    ConfigEntry<bool, documentation::HideScrollbarInAltScreen> hideScrollbarInAltScreen { true };
-    ConfigEntry<bool, documentation::Dummy> optionKeyAsAlt { false };
-    ConfigEntry<bool, documentation::AutoScrollOnUpdate> autoScrollOnUpdate { true };
+    ConfigEntry<WindowMargins, documentation::Margins> margins { { HorizontalMargin { 0u },
+                                                                   VerticalMargin { 0u } } };
+    ConfigEntry<HistoryConfig, documentation::History> history {};
+    ConfigEntry<ScrollBarConfig, documentation::Scrollbar> scrollbar {};
+    ConfigEntry<MouseConfig, documentation::Mouse> mouse { true };
+    ConfigEntry<PermissionsConfig, documentation::Permissions> permissions {};
+    ConfigEntry<bool, documentation::HighlightDoubleClickerWord> highlightDoubleClickedWord { true };
     ConfigEntry<vtrasterizer::FontDescriptions, documentation::Fonts> fonts { defaultFont };
-    ConfigEntry<Permission, documentation::CaptureBuffer> captureBuffer { Permission::Ask };
-    ConfigEntry<Permission, documentation::ChangeFont> changeFont { Permission::Ask };
-    ConfigEntry<Permission, documentation::DisplayHostWritableStatusLine> displayHostWritableStatusLine {
-        Permission::Ask
-    };
     ConfigEntry<bool, documentation::DrawBoldTextWithBrightColors> drawBoldTextWithBrightColors { false };
-    ConfigEntry<ColorConfig, documentation::Colors> colors { SimpleColorConfig {} };
-    ConfigEntry<vtbackend::LineCount, documentation::ModalCursorScrollOff> modalCursorScrollOff {
-        vtbackend::LineCount { 8 }
-    };
     ConfigEntry<InputModeConfig, documentation::ModeInsert> modeInsert { CursorConfig {
         vtbackend::CursorShape::Bar, vtbackend::CursorDisplay::Steady, std::chrono::milliseconds { 500 } } };
     ConfigEntry<InputModeConfig, documentation::ModeNormal> modeNormal {
@@ -381,23 +433,17 @@ struct TerminalProfile
                        vtbackend::CursorDisplay::Steady,
                        std::chrono::milliseconds { 500 } },
     };
-    ConfigEntry<std::chrono::milliseconds, documentation::SmoothLineScrolling> smoothLineScrolling { 100 };
     ConfigEntry<std::chrono::milliseconds, documentation::HighlightTimeout> highlightTimeout { 100 };
-    ConfigEntry<bool, documentation::HighlightDoubleClickerWord> highlightDoubleClickedWord { true };
-    ConfigEntry<vtbackend::StatusDisplayType, documentation::InitialStatusLine> initialStatusDisplayType {
-        vtbackend::StatusDisplayType::Indicator
+    ConfigEntry<vtbackend::LineCount, documentation::ModalCursorScrollOff> modalCursorScrollOff {
+        vtbackend::LineCount { 8 }
     };
-    ConfigEntry<vtbackend::Opacity, documentation::BackgroundOpacity> backgroundOpacity { vtbackend::Opacity(
-        0xFF) };
-    ConfigEntry<bool, documentation::BackgroundBlur> backgroundBlur { false };
-    ConfigEntry<vtrasterizer::Decorator, "normal: {}\n"> hyperlinkDecorationNormal {
-        vtrasterizer::Decorator::DottedUnderline
-    };
-    ConfigEntry<vtrasterizer::Decorator, "hover: {}\n"> hyperlinkDecorationHover {
-        vtrasterizer::Decorator::Underline
-    };
-    ConfigEntry<Bell, documentation::Bell> bell { { .sound = "default", .alert = true, .volume = 1.0f } };
-    ConfigEntry<std::map<vtbackend::DECMode, bool>, documentation::FrozenDecMode> frozenModes {};
+    ConfigEntry<StatusLineConfig, documentation::StatusLine> statusLine {};
+    ConfigEntry<BackgroundConfig, documentation::Background> background {};
+    ConfigEntry<ColorConfig, documentation::Colors> colors { SimpleColorConfig {} };
+    ConfigEntry<HyperlinkDecorationConfig, documentation::HyperlinkDecoration> hyperlinkDecoration {};
+
+    ConfigEntry<std::string, documentation::WMClass> wmClass { CONTOUR_APP_ID };
+    ConfigEntry<bool, documentation::OptionKeyAsAlt> optionKeyAsAlt { false };
 };
 
 const InputMappings defaultInputMappings {
@@ -442,6 +488,14 @@ const InputMappings defaultInputMappings {
                           .modifiers { vtbackend::Modifiers { vtbackend::Modifier::Shift } },
                           .input = vtbackend::Key::F3,
                           .binding = { { actions::FocusPreviousSearchMatch {} } } },
+        KeyInputMapping { .modes { vtbackend::MatchModes {} },
+                          .modifiers { vtbackend::Modifier::Shift },
+                          .input = vtbackend::Key::LeftArrow,
+                          .binding = { { actions::SwitchToTabLeft {} } } },
+        KeyInputMapping { .modes { vtbackend::MatchModes {} },
+                          .modifiers { vtbackend::Modifier::Shift },
+                          .input = vtbackend::Key::RightArrow,
+                          .binding = { { actions::SwitchToTabRight {} } } },
         //     KeyInputMapping { .modes { vtbackend::MatchModes {} },
         //                       .modifiers { vtbackend::Modifiers {  vtbackend::Modifier {
         //                       vtbackend::Modifier::Shift }
@@ -482,12 +536,6 @@ const InputMappings defaultInputMappings {
                                                 | vtbackend::Modifiers { vtbackend::Modifier::Control } } },
             .input = 'V',
             .binding = { { actions::PasteClipboard { .strip = false } } } },
-        CharInputMapping {
-            .modes { vtbackend::MatchModes {} },
-            .modifiers { vtbackend::Modifiers { vtbackend::Modifiers { vtbackend::Modifier::Shift }
-                                                | vtbackend::Modifiers { vtbackend::Modifier::Control } } },
-            .input = 'V',
-            .binding = { { actions::PasteClipboard { .strip = false } } } },
         CharInputMapping { .modes { vtbackend::MatchModes {} },
                            .modifiers { vtbackend::Modifiers { vtbackend::Modifier::Alt }
                                         | vtbackend::Modifiers { vtbackend::Modifier::Control } },
@@ -497,6 +545,50 @@ const InputMappings defaultInputMappings {
                            .modifiers { vtbackend::Modifiers { vtbackend::Modifier::Control } },
                            .input = '0',
                            .binding = { { actions::ResetFontSize {} } } },
+        CharInputMapping { .modes { vtbackend::MatchModes {} },
+                           .modifiers { vtbackend::Modifier::Control, vtbackend::Modifier::Shift },
+                           .input = 'T',
+                           .binding = { { actions::CreateNewTab {} } } },
+        CharInputMapping { .modes { vtbackend::MatchModes {} },
+                           .modifiers { vtbackend::Modifier::Alt },
+                           .input = '1',
+                           .binding = { { actions::SwitchToTab { .position = 1 } } } },
+        CharInputMapping { .modes { vtbackend::MatchModes {} },
+                           .modifiers { vtbackend::Modifier::Alt },
+                           .input = '2',
+                           .binding = { { actions::SwitchToTab { .position = 2 } } } },
+        CharInputMapping { .modes { vtbackend::MatchModes {} },
+                           .modifiers { vtbackend::Modifier::Alt },
+                           .input = '3',
+                           .binding = { { actions::SwitchToTab { .position = 3 } } } },
+        CharInputMapping { .modes { vtbackend::MatchModes {} },
+                           .modifiers { vtbackend::Modifier::Alt },
+                           .input = '4',
+                           .binding = { { actions::SwitchToTab { .position = 4 } } } },
+        CharInputMapping { .modes { vtbackend::MatchModes {} },
+                           .modifiers { vtbackend::Modifier::Alt },
+                           .input = '5',
+                           .binding = { { actions::SwitchToTab { .position = 5 } } } },
+        CharInputMapping { .modes { vtbackend::MatchModes {} },
+                           .modifiers { vtbackend::Modifier::Alt },
+                           .input = '6',
+                           .binding = { { actions::SwitchToTab { .position = 6 } } } },
+        CharInputMapping { .modes { vtbackend::MatchModes {} },
+                           .modifiers { vtbackend::Modifier::Alt },
+                           .input = '7',
+                           .binding = { { actions::SwitchToTab { .position = 7 } } } },
+        CharInputMapping { .modes { vtbackend::MatchModes {} },
+                           .modifiers { vtbackend::Modifier::Alt },
+                           .input = '8',
+                           .binding = { { actions::SwitchToTab { .position = 8 } } } },
+        CharInputMapping { .modes { vtbackend::MatchModes {} },
+                           .modifiers { vtbackend::Modifier::Alt },
+                           .input = '9',
+                           .binding = { { actions::SwitchToTab { .position = 9 } } } },
+        CharInputMapping { .modes { vtbackend::MatchModes {} },
+                           .modifiers { vtbackend::Modifier::Alt },
+                           .input = '0',
+                           .binding = { { actions::SwitchToTab { .position = 10 } } } },
         CharInputMapping { .modes = []() -> vtbackend::MatchModes {
                               auto mods = vtbackend::MatchModes();
                               mods.enable(vtbackend::MatchModes::Select);
@@ -647,48 +739,40 @@ const InputMappings defaultInputMappings {
 struct Config
 {
     std::filesystem::path configFile {};
-    ConfigEntry<bool, documentation::Live> live { false };
     ConfigEntry<std::string, documentation::PlatformPlugin> platformPlugin { "auto" };
-    ConfigEntry<RenderingBackend, documentation::RenderingBackend> renderingBackend {
-        RenderingBackend::Default
-    };
-    ConfigEntry<bool, documentation::TextureAtlasDirectMapping> textureAtlasDirectMapping { false };
-    ConfigEntry<crispy::strong_hashtable_size, documentation::TextureAtlasHashtableSlots>
-        textureAtlasHashtableSlots { 4096u };
-    ConfigEntry<crispy::lru_capacity, documentation::TextureAtlasTileCount> textureAtlasTileCount { 4000u };
-    ConfigEntry<int, documentation::PTYReadBufferSize> ptyReadBufferSize { 16384 };
-    ConfigEntry<int, documentation::PTYBufferObjectSize> ptyBufferObjectSize { 1024 * 1024 };
-    ConfigEntry<bool, documentation::ReflowOnResize> reflowOnResize { true };
-    ConfigEntry<std::unordered_map<std::string, vtbackend::ColorPalette>, documentation::ColorSchemes>
-        colorschemes { { { "default", vtbackend::ColorPalette {} } } };
-    ConfigEntry<std::unordered_map<std::string, TerminalProfile>, documentation::Profiles> profiles {
-        { { "main", TerminalProfile {} } }
-    };
-    ConfigEntry<std::string, "default_profile: {}\n"> defaultProfileName { "main" };
+    ConfigEntry<RendererConfig, documentation::Renderer> renderer {};
     ConfigEntry<std::string, documentation::WordDelimiters> wordDelimiters {
         " /\\()\"'-.,:;<>~!@#$%^&*+=[]{{}}~?|│"
     };
     ConfigEntry<std::string, documentation::ExtendedWordDelimiters> extendedWordDelimiters {
         " /\\()\"'-.,:;<>~!@#$%^&*+=[]{{}}~?|│"
     };
-
-    ConfigEntry<vtbackend::Modifiers, documentation::BypassMouseProtocolModifiers>
-        bypassMouseProtocolModifiers { vtbackend::Modifier::Shift };
-    ConfigEntry<contour::config::SelectionAction, documentation::OnMouseSelection> onMouseSelection {
-        contour::config::SelectionAction::CopyToSelectionClipboard
-    };
-    ConfigEntry<vtbackend::Modifiers, documentation::MouseBlockSelectionModifiers>
-        mouseBlockSelectionModifiers { vtbackend::Modifier::Control };
-    ConfigEntry<InputMappings, documentation::InputMappings> inputMappings { defaultInputMappings };
+    ConfigEntry<int, documentation::PTYReadBufferSize> ptyReadBufferSize { 16384 };
+    ConfigEntry<int, documentation::PTYBufferObjectSize> ptyBufferObjectSize { 1024 * 1024 };
+    ConfigEntry<std::string, documentation::DefaultProfiles> defaultProfileName { "main" };
     ConfigEntry<unsigned, documentation::EarlyExitThreshold> earlyExitThreshold {
         documentation::DefaultEarlyExitThreshold
     };
     ConfigEntry<bool, documentation::SpawnNewProcess> spawnNewProcess { false };
-    ConfigEntry<bool, documentation::SixelScrolling> sixelScrolling { true };
-    ConfigEntry<vtbackend::ImageSize, documentation::MaxImageSize> maxImageSize { { vtpty::Width { 0 },
-                                                                                    vtpty::Height { 0 } } };
-    ConfigEntry<int, documentation::MaxImageColorRegisters> maxImageColorRegisters { 4096 };
+    ConfigEntry<bool, documentation::ReflowOnResize> reflowOnResize { true };
+    ConfigEntry<vtbackend::Modifiers, documentation::BypassMouseProtocolModifiers>
+        bypassMouseProtocolModifiers { vtbackend::Modifier::Shift };
+    ConfigEntry<vtbackend::Modifiers, documentation::MouseBlockSelectionModifiers>
+        mouseBlockSelectionModifiers { vtbackend::Modifier::Control };
+    ConfigEntry<contour::config::SelectionAction, documentation::OnMouseSelection> onMouseSelection {
+        contour::config::SelectionAction::CopyToSelectionClipboard
+    };
+    ConfigEntry<bool, documentation::Live> live { false };
     ConfigEntry<std::set<std::string>, documentation::ExperimentalFeatures> experimentalFeatures {};
+    ConfigEntry<ImagesConfig, documentation::Images> images {};
+
+    ConfigEntry<std::unordered_map<std::string, TerminalProfile>, documentation::Profiles> profiles {
+        { { "main", TerminalProfile {} } }
+    };
+    ConfigEntry<std::unordered_map<std::string, vtbackend::ColorPalette>, documentation::ColorSchemes>
+        colorschemes { { { "default", vtbackend::ColorPalette {} } } };
+
+    ConfigEntry<InputMappings, documentation::InputMappings> inputMappings { defaultInputMappings };
 
     TerminalProfile* profile(std::string const& name) noexcept
     {
@@ -742,8 +826,10 @@ struct YAMLConfigReader
         }
     }
 
-    template <typename T, documentation::StringLiteral D>
-    void loadFromEntry(YAML::Node const& node, std::string const& entry, ConfigEntry<T, D>& where)
+    template <typename T, documentation::StringLiteral ConfigDoc, documentation::StringLiteral WebDoc>
+    void loadFromEntry(YAML::Node const& node,
+                       std::string const& entry,
+                       ConfigEntry<T, documentation::DocumentationEntry<ConfigDoc, WebDoc>>& where)
     {
         try
         {
@@ -755,8 +841,13 @@ struct YAMLConfigReader
         }
     }
 
-    template <typename T, documentation::StringLiteral D, typename... Args>
-    void loadFromEntry(std::string const& entry, ConfigEntry<T, D>& where, Args&&... args)
+    template <typename T,
+              documentation::StringLiteral ConfigDoc,
+              documentation::StringLiteral WebDoc,
+              typename... Args>
+    void loadFromEntry(std::string const& entry,
+                       ConfigEntry<T, documentation::DocumentationEntry<ConfigDoc, WebDoc>>& where,
+                       Args&&... args)
     {
         loadFromEntry(doc, entry, where.value(), std::forward<Args>(args)...);
     }
@@ -894,6 +985,17 @@ struct YAMLConfigReader
     void loadFromEntry(YAML::Node const& node, std::string const& entry, vtbackend::ColorPalette& where);
     void loadFromEntry(YAML::Node const& node, vtbackend::ColorPalette& where);
     void loadFromEntry(YAML::Node const& node, std::string const& entry, TerminalProfile& where);
+    void loadFromEntry(YAML::Node const& node, std::string const& entry, RendererConfig& where);
+    void loadFromEntry(YAML::Node const& node, std::string const& entry, ImagesConfig& where);
+    void loadFromEntry(YAML::Node const& node, std::string const& entry, HistoryConfig& where);
+    void loadFromEntry(YAML::Node const& node, std::string const& entry, ScrollBarConfig& where);
+    void loadFromEntry(YAML::Node const& node, std::string const& entry, MouseConfig& where);
+    void loadFromEntry(YAML::Node const& node, std::string const& entry, StatusLineConfig& where);
+    void loadFromEntry(YAML::Node const& node, std::string const& entry, BackgroundConfig& where);
+    void loadFromEntry(YAML::Node const& node, std::string const& entry, HyperlinkDecorationConfig& where);
+    void loadFromEntry(YAML::Node const& node, std::string const& entry, PermissionsConfig& where);
+
+
     void defaultSettings(vtpty::Process::ExecInfo& shell);
     // clang-format on
 
@@ -937,47 +1039,12 @@ struct Writer
         lambda();
     }
 
-    template <typename T>
-    auto format(T v)
-    {
-        return std::format("{}", v);
-    }
+    virtual inline std::string replaceCommentPlaceholder(std::string const& docString) = 0;
 
     template <typename... T>
     [[nodiscard]] std::string format(std::string_view doc, T... args)
     {
-        return std::vformat(helper::replaceCommentPlaceholder(std::string(doc)),
-                            std::make_format_args(args...));
-    }
-};
-
-template <typename T>
-std::string createString(Config const& c, T)
-{
-    return createString<T>(c);
-}
-
-struct YAMLConfigWriter: Writer
-{
-
-    static constexpr int OneOffset = 4;
-    using Writer::format;
-    std::string static addOffset(std::string_view doc, size_t off)
-    {
-        auto offset = std::string(off, ' ');
-        return std::regex_replace(std::string { doc }, std::regex(".+\n"), offset + "$&");
-    }
-
-    template <typename T>
-    std::string process(std::string_view doc, T val)
-    {
-        return format(addOffset(doc, Offset::levels * OneOffset), val);
-    }
-
-    template <typename... T>
-    std::string process(std::string_view doc, T... val)
-    {
-        return format(addOffset(doc, Offset::levels * OneOffset), val...);
+        return std::vformat(replaceCommentPlaceholder(std::string(doc)), std::make_format_args(args...));
     }
 
     [[nodiscard]] std::string format(KeyInputMapping v)
@@ -1069,7 +1136,29 @@ struct YAMLConfigWriter: Writer
         auto args = std::string { "[" };
         args.append(v.arguments | ranges::views::join(", ") | ranges::to<std::string>);
         args.append("]");
-        return format(doc, v.program, args);
+        return format(doc, v.program, args, [&]() -> std::string {
+            auto fromConfig = v.workingDirectory.string();
+            if (fromConfig.empty()
+                || fromConfig == crispy::homeResolvedPath("~", vtpty::Process::homeDirectory()))
+                return std::string { "\"~\"" };
+            return fromConfig;
+        }());
+    }
+
+    [[nodiscard]] std::string format(std::string_view doc, std::map<vtbackend::DECMode, bool> const& mods)
+    {
+        if (mods.empty())
+            return format(doc, std::string_view { "0" });
+
+        auto result = std::string { "[" };
+        for (auto const& [mode, value]: mods)
+        {
+            if (!result.empty())
+                result += ',';
+            result += std::format("{}", to_string(mode));
+        }
+        result += "]";
+        return format(doc, result);
     }
 
     [[nodiscard]] std::string format(std::string_view doc, vtpty::SshHostConfig const& v)
@@ -1087,14 +1176,6 @@ struct YAMLConfigWriter: Writer
 
     [[nodiscard]] std::string static format(vtbackend::RGBColor const& v) { return std::format("'{}'", v); }
 
-    [[nodiscard]] std::string format(std::string_view doc, vtbackend::MaxHistoryLineCount v)
-    {
-        if (std::holds_alternative<vtbackend::Infinite>(v))
-            return format(doc, -1);
-        auto number = unbox(std::get<vtbackend::LineCount>(v));
-        return format(doc, number);
-    }
-
     [[nodiscard]] std::string format(std::string_view doc, vtbackend::ImageSize v)
     {
         return format(doc, unbox(v.width), unbox(v.height));
@@ -1111,11 +1192,11 @@ struct YAMLConfigWriter: Writer
             return format(doc, simple->colorScheme);
         else if (auto* dual = get_if<DualColorConfig>(&v))
         {
-            auto const formattedValue = std::format("\n"
-                                                    "    light: {}\n"
-                                                    "    dark: {}\n",
-                                                    dual->colorSchemeLight,
-                                                    dual->colorSchemeDark);
+            auto const formattedValue = format("\n"
+                                               "    light: {}\n"
+                                               "    dark: {}\n",
+                                               dual->colorSchemeLight,
+                                               dual->colorSchemeDark);
             return format(doc, formattedValue);
         }
 
@@ -1127,11 +1208,76 @@ struct YAMLConfigWriter: Writer
         return format(doc, v.sound, v.volume, v.alert);
     }
 
+    [[nodiscard]] std::string format(std::string_view doc, RendererConfig& v)
+    {
+        return format(doc,
+                      v.renderingBackend,
+                      v.textureAtlasDirectMapping,
+                      v.textureAtlasHashtableSlots,
+                      v.textureAtlasTileCount);
+    }
+
+    [[nodiscard]] std::string format(std::string_view doc, ImagesConfig& v)
+    {
+        return format(
+            doc, v.sixelScrolling, v.maxImageColorRegisters, v.maxImageSize.width, v.maxImageSize.height);
+    }
+
     [[nodiscard]] std::string format(std::string_view doc, WindowMargins& v)
     {
         return format(doc, v.horizontal, v.vertical);
     }
 
+    [[nodiscard]] std::string format(std::string_view doc, HistoryConfig& v)
+    {
+        return format(
+            doc,
+            [&]() {
+                if (std::holds_alternative<vtbackend::Infinite>(v.maxHistoryLineCount))
+                    return -1;
+                auto number = unbox(std::get<vtbackend::LineCount>(v.maxHistoryLineCount));
+                return number;
+            }(),
+            v.autoScrollOnUpdate,
+            v.historyScrollMultiplier);
+    }
+
+    [[nodiscard]] std::string format(std::string_view doc, ScrollBarConfig& v)
+    {
+        return format(doc, v.position, v.hideScrollbarInAltScreen);
+    }
+
+    [[nodiscard]] std::string format(std::string_view doc, MouseConfig& v)
+    {
+        return format(doc, v.hideWhileTyping);
+    }
+
+    [[nodiscard]] std::string format(std::string_view doc, StatusLineConfig& v)
+    {
+        return format(doc,
+                      v.initialType,
+                      v.position,
+                      v.syncWindowTitleWithHostWritableStatusDisplay,
+                      v.indicator.left,
+                      v.indicator.middle,
+                      v.indicator.right);
+    }
+
+    [[nodiscard]] std::string format(std::string_view doc, BackgroundConfig& v)
+    {
+        return format(doc, v.opacity, v.blur);
+    }
+
+    [[nodiscard]] std::string format(std::string_view doc, HyperlinkDecorationConfig& v)
+    {
+        return format(doc, v.normal, v.hover);
+    }
+
+    [[nodiscard]] std::string format(std::string_view doc, PermissionsConfig& v)
+    {
+        return format(doc, v.captureBuffer, v.changeFont, v.displayHostWritableStatusLine);
+    }
+
     [[nodiscard]] std::string format(std::string_view doc, InputModeConfig v)
     {
         auto const shape = [v]() -> std::string_view {
@@ -1150,6 +1296,144 @@ struct YAMLConfigWriter: Writer
     }
 };
 
+template <typename T>
+std::string createString(Config const& c);
+
+template <typename T>
+std::string documentationGlobalConfig(Config const& c);
+
+template <typename T>
+std::string documentationProfileConfig(Config const& c);
+
+struct YAMLConfigWriter: Writer
+{
+
+    constexpr static std::string_view FormatTemplate = "{}";
+    inline std::string replaceCommentPlaceholder(std::string const& docString) override
+    {
+        return std::regex_replace(docString, std::regex { "\\{comment\\}" }, "#");
+    }
+
+    static constexpr int OneOffset = 4;
+    using Writer::format;
+    std::string static addOffset(std::string_view doc, size_t off)
+    {
+        auto offset = std::string(off, ' ');
+        return std::regex_replace(std::string { doc }, std::regex(".+\n"), offset + "$&");
+    }
+
+    template <typename... T>
+    std::string process(std::string_view doc, T... val)
+    {
+        return format(addOffset(replaceCommentPlaceholder(std::string { doc }), Offset::levels * OneOffset),
+                      val...);
+    }
+
+    template <typename... T>
+    std::string process(std::string_view doc, [[maybe_unused]] std::string_view name, T... val)
+    {
+        return format(addOffset(replaceCommentPlaceholder(std::string { doc }), Offset::levels * OneOffset),
+                      val...);
+    }
+
+    template <typename T, documentation::StringLiteral ConfigDoc, documentation::StringLiteral WebDoc>
+    constexpr std::string_view whichDoc(
+        contour::config::
+            ConfigEntry<T, contour::config::documentation::DocumentationEntry<ConfigDoc, WebDoc>> const&)
+    {
+        return ConfigDoc.value;
+    }
+
+    template <documentation::StringLiteral ConfigDoc, documentation::StringLiteral WebDoc>
+    constexpr std::string_view whichDoc(
+        contour::config::documentation::DocumentationEntry<ConfigDoc, WebDoc> const&)
+    {
+        return ConfigDoc.value;
+    }
+};
+
+struct DocumentationWriter: Writer
+{
+    constexpr static std::string_view FormatTemplate = "{}";
+    inline std::string replaceCommentPlaceholder(std::string const& docString) override
+    {
+        return std::regex_replace(docString, std::regex { "\\{comment\\}" }, "");
+    }
+
+    static constexpr int OneOffset = 0;
+    std::string static addOffset(std::string_view doc, [[maybe_unused]] size_t off)
+    {
+        return std::string { doc };
+    }
+
+    using Writer::format;
+    template <typename... T>
+    std::string process(std::string_view doc, T... val)
+    {
+        return process(doc, std::string_view { "" }, val...);
+    }
+
+    template <typename... T>
+    std::string process(std::string_view doc, std::string_view name, T... val)
+    {
+        return format(
+            "### `{}`\n"
+            "{}\n",
+            [](std::string_view name) -> std::string {
+                // camelCase into snake_case
+                auto result = std::string { name };
+                for (auto i = 0u; i < result.size(); ++i)
+                {
+                    if (std::isupper(result[i]))
+                    {
+                        result.insert(i, 1, '_');
+                        result[i + 1] = static_cast<char>(std::tolower(result[i + 1]));
+                    }
+                }
+                return result;
+            }(name),
+            format(replaceCommentPlaceholder(std::string { doc }), val...));
+    }
+
+    template <typename T, documentation::StringLiteral ConfigDoc, documentation::StringLiteral WebDoc>
+    constexpr std::string_view whichDoc(
+        contour::config::
+            ConfigEntry<T, contour::config::documentation::DocumentationEntry<ConfigDoc, WebDoc>> const&)
+    {
+        return WebDoc.value;
+    }
+
+    template <documentation::StringLiteral ConfigDoc, documentation::StringLiteral WebDoc>
+    constexpr std::string_view whichDoc(
+        contour::config::documentation::DocumentationEntry<ConfigDoc, WebDoc> const&)
+    {
+        return WebDoc.value;
+    }
+};
+
+// Will ignore documentation
+struct PlainWriter: Writer
+{
+    constexpr static std::string_view FormatTemplate = "{}";
+    inline std::string replaceCommentPlaceholder(std::string const& docString) override
+    {
+        return std::regex_replace(docString, std::regex { "\\{comment\\}" }, "");
+    }
+
+    static constexpr int OneOffset = 0;
+    std::string static addOffset(std::string_view doc, [[maybe_unused]] size_t off)
+    {
+        return std::string { doc };
+    }
+
+    using Writer::format;
+    template <typename... T>
+    std::string process([[maybe_unused]] std::string_view doc, T... val)
+    {
+        return format("{}", format(val...));
+    }
+};
+
 std::filesystem::path configHome();
 std::filesystem::path configHome(std::string const& programName);
 
@@ -1164,6 +1448,9 @@ std::string defaultConfigString();
 std::error_code createDefaultConfig(std::filesystem::path const& path);
 std::string defaultConfigFilePath();
 
+std::string documentationGlobalConfig();
+std::string documentationProfileConfig();
+
 } // namespace contour::config
 
 // {{{ fmtlib custom formatter support
@@ -1325,10 +1612,16 @@ struct std::formatter<contour::config::WindowMargins>: public std::formatter<std
     }
 };
 
-template <typename T, contour::config::documentation::StringLiteral D>
-struct std::formatter<contour::config::ConfigEntry<T, D>>
+template <typename T,
+          contour::config::documentation::StringLiteral ConfigDoc,
+          contour::config::documentation::StringLiteral WebDoc>
+struct std::formatter<
+    contour::config::ConfigEntry<T, contour::config::documentation::DocumentationEntry<ConfigDoc, WebDoc>>>
 {
-    auto format(contour::config::ConfigEntry<T, D> const& c, auto& ctx) const
+    auto format(contour::config::ConfigEntry<
+                    T,
+                    contour::config::documentation::DocumentationEntry<ConfigDoc, WebDoc>> const& c,
+                auto& ctx) const
     {
         return std::format_to(ctx.out(), "{}", c.value());
     }
