@@ -70,6 +70,7 @@ crispy::cli::command ContourGuiApp::parameterDefinition() const
 {
     auto command = ContourApp::parameterDefinition();
 
+    // NOLINTBEGIN
     command.children.insert(
         command.children.begin(),
         CLI::command {
@@ -147,6 +148,7 @@ crispy::cli::command ContourGuiApp::parameterDefinition() const
             CLI::verbatim { "PROGRAM ARGS...",
                             "Executes given program instead of the one provided in the configuration." } });
 
+    // NOLINTEND
     return command;
 }
 
@@ -343,7 +345,11 @@ int ContourGuiApp::terminalGuiAction()
     if (!loadConfig("terminal"))
         return EXIT_FAILURE;
 
-    switch (_config.renderingBackend.value())
+#if defined(__APPLE__)
+    QGuiApplication::setAttribute(Qt::AA_MacDontSwapCtrlAndMeta, true);
+#endif
+
+    switch (_config.renderer.value().renderingBackend)
     {
         case config::RenderingBackend::OpenGL:
             QGuiApplication::setAttribute(Qt::AA_UseSoftwareOpenGL, false);
@@ -366,6 +372,8 @@ int ContourGuiApp::terminalGuiAction()
     QCoreApplication::setApplicationName(appName);
     QCoreApplication::setOrganizationName("contour");
     QCoreApplication::setApplicationVersion(CONTOUR_VERSION_STRING);
+    // Expose the profile name to the shell.
+    _config.profile(profileName())->shell.value().env["CONTOUR_PROFILE"] = profileName();
 
     vector<string> qtArgsStore;
     vector<char const*> qtArgsPtr;
@@ -519,7 +527,7 @@ void ContourGuiApp::setupQCoreApplication()
         return defaultAppName;
     }();
 
-    // On Wayland, we want to set the Applicion Id to the configured value via the desktop file name.
+    // On Wayland, we want to set the Application Id to the configured value via the desktop file name.
     // We use the desktop file name as the application id, because that's what Qt uses to set the
     // app id on Wayland.
     // I know this sounds weird. This is because it is weird. But it's the only way to set the
@@ -553,6 +561,7 @@ void ContourGuiApp::ensureTermInfoFile()
 void ContourGuiApp::newWindow()
 {
     _qmlEngine->load(resolveResource("ui/main.qml"));
+    _sessionManager.display = _sessionManager.getSession()->display();
 }
 
 void ContourGuiApp::showNotification(std::string_view title, std::string_view content)
