@@ -1,4 +1,5 @@
 // SPDX-License-Identifier: Apache-2.0
+#include <contour/Actions.h>
 #include <contour/ContourGuiApp.h>
 #include <contour/TerminalSession.h>
 #include <contour/display/TerminalDisplay.h>
@@ -132,7 +133,7 @@ namespace
         settings.pageSize = profile.terminalSize.value();
         settings.ptyBufferObjectSize = config.ptyBufferObjectSize.value();
         settings.ptyReadBufferSize = config.ptyReadBufferSize.value();
-        settings.maxHistoryLineCount = profile.maxHistoryLineCount.value();
+        settings.maxHistoryLineCount = profile.history.value().maxHistoryLineCount;
         settings.copyLastMarkRangeOffset = profile.copyLastMarkRangeOffset.value();
         settings.cursorBlinkInterval = profile.modeInsert.value().cursor.cursorBlinkInterval;
         settings.cursorShape = profile.modeInsert.value().cursor.cursorShape;
@@ -140,15 +141,15 @@ namespace
         settings.smoothLineScrolling = profile.smoothLineScrolling.value();
         settings.wordDelimiters = unicode::from_utf8(config.wordDelimiters.value());
         settings.mouseProtocolBypassModifiers = config.bypassMouseProtocolModifiers.value();
-        settings.maxImageSize = config.maxImageSize.value();
-        settings.maxImageRegisterCount = config.maxImageColorRegisters.value();
-        settings.statusDisplayType = profile.initialStatusDisplayType.value();
-        settings.statusDisplayPosition = profile.statusDisplayPosition.value();
-        settings.indicatorStatusLine.left = profile.indicatorStatusLineLeft.value();
-        settings.indicatorStatusLine.middle = profile.indicatorStatusLineMiddle.value();
-        settings.indicatorStatusLine.right = profile.indicatorStatusLineRight.value();
+        settings.maxImageSize = config.images.value().maxImageSize;
+        settings.maxImageRegisterCount = config.images.value().maxImageColorRegisters;
+        settings.statusDisplayType = profile.statusLine.value().initialType;
+        settings.statusDisplayPosition = profile.statusLine.value().position;
+        settings.indicatorStatusLine.left = profile.statusLine.value().indicator.left;
+        settings.indicatorStatusLine.middle = profile.statusLine.value().indicator.middle;
+        settings.indicatorStatusLine.right = profile.statusLine.value().indicator.right;
         settings.syncWindowTitleWithHostWritableStatusDisplay =
-            profile.syncWindowTitleWithHostWritableStatusDisplay.value();
+            profile.statusLine.value().syncWindowTitleWithHostWritableStatusDisplay;
         if (auto const* p = preferredColorPalette(profile.colors.value(), colorPreference))
             settings.colorPalette = *p;
         settings.primaryScreen.allowReflowOnResize = config.reflowOnResize.value();
@@ -235,20 +236,15 @@ void TerminalSession::detachDisplay(display::TerminalDisplay& display)
 
 void TerminalSession::attachDisplay(display::TerminalDisplay& newDisplay)
 {
-    sessionLog()("Attaching display.");
-    // newDisplay.setSession(*this); // NB: we're being called by newDisplay!
-    _display = &newDisplay;
+    sessionLog()("Attaching session to display {}x{}.", newDisplay.width(), newDisplay.height());
 
-    setContentScale(newDisplay.contentScale());
+    // We're being called by newDisplay!
+    _display = &newDisplay;
 
     {
         // NB: Inform connected TTY and local Screen instance about initial cell pixel size.
-        auto const pixels = _display->cellSize() * _terminal.pageSize();
-        // auto const pixels =
-        //     ImageSize { _display->cellSize().width * boxed_cast<Width>(_terminal.pageSize().columns),
-        //                 _display->cellSize().height * boxed_cast<Height>(_terminal.pageSize().lines) };
         auto const l = scoped_lock { _terminal };
-        _terminal.resizeScreen(_terminal.pageSize(), pixels);
+        _terminal.resizeScreen(_terminal.pageSize(), _display->pixelSize());
         _terminal.setRefreshRate(_display->refreshRate());
     }
 
@@ -268,10 +264,14 @@ void TerminalSession::scheduleRedraw()
 
 void TerminalSession::start()
 {
-    sessionLog()("Starting terminal session.");
-    _terminal.device().start();
-    _screenUpdateThread = make_unique<std::thread>(bind(&TerminalSession::mainLoop, this));
-    _exitWatcherThread->start(QThread::LowPriority);
+    // ensure that we start only once
+    if (!_screenUpdateThread)
+    {
+        sessionLog()("Starting terminal session.");
+        _terminal.device().start();
+        _screenUpdateThread = make_unique<std::thread>(bind(&TerminalSession::mainLoop, this));
+        _exitWatcherThread->start(QThread::LowPriority);
+    }
 }
 
 void TerminalSession::mainLoop()
@@ -328,7 +328,7 @@ void TerminalSession::screenUpdated()
     if (!_display)
         return;
 
-    if (_profile.autoScrollOnUpdate.value() && terminal().viewport().scrolled()
+    if (_profile.history.value().autoScrollOnUpdate && terminal().viewport().scrolled()
         && terminal().inputHandler().mode() == ViMode::Insert)
         terminal().viewport().scrollToBottom();
 
@@ -368,6 +368,7 @@ void TerminalSession::executeRole(GuardedRole role, bool allow, bool remember)
         case GuardedRole::ShowHostWritableStatusLine:
             executeShowHostWritableStatusLine(allow, remember);
             break;
+        case GuardedRole::BigPaste: applyPendingPaste(allow, remember); break;
     }
 }
 
@@ -401,6 +402,7 @@ void TerminalSession::requestPermission(config::Permission allowedByConfig, Guar
                     case GuardedRole::ChangeFont: emit requestPermissionForFontChange(); break;
                     case GuardedRole::CaptureBuffer: emit requestPermissionForBufferCapture(); break;
                     case GuardedRole::ShowHostWritableStatusLine: emit requestPermissionForShowHostWritableStatusLine(); break;
+                    case GuardedRole::BigPaste: emit requestPermissionForPasteLargeFile(); break;
                         // clang-format on
                 }
             }
@@ -428,7 +430,7 @@ void TerminalSession::requestCaptureBuffer(LineCount lines, bool logical)
     if (!_display)
         return;
 
-    _pendingBufferCapture = CaptureBufferRequest { lines, logical };
+    _pendingBufferCapture = CaptureBufferRequest { .lines = lines, .logical = logical };
 
     emit requestPermissionForBufferCapture();
     // _display->post(
@@ -459,7 +461,7 @@ void TerminalSession::requestShowHostWritableStatusLine()
 {
     if (_display)
         _display->post([this]() {
-            requestPermission(_profile.displayHostWritableStatusLine.value(),
+            requestPermission(_profile.permissions.value().displayHostWritableStatusLine,
                               GuardedRole::ShowHostWritableStatusLine);
         });
 }
@@ -490,7 +492,8 @@ void TerminalSession::setFontDef(vtbackend::FontDef const& fontDef)
 
     _pendingFontChange = fontDef;
 
-    _display->post([this]() { requestPermission(_profile.changeFont.value(), GuardedRole::ChangeFont); });
+    _display->post(
+        [this]() { requestPermission(_profile.permissions.value().changeFont, GuardedRole::ChangeFont); });
 }
 
 void TerminalSession::applyPendingFontChange(bool allow, bool remember)
@@ -544,7 +547,7 @@ void TerminalSession::copyToClipboard(std::string_view data)
     if (!_display)
         return;
 
-    _display->post([this, data = string(data)]() { _display->copyToClipboard(data); });
+    _display->post([data = string(data)]() { display::TerminalDisplay::copyToClipboard(data); });
 }
 
 void TerminalSession::openDocument(std::string_view fileOrUrl)
@@ -650,16 +653,38 @@ void TerminalSession::pasteFromClipboard(unsigned count, bool strip)
         sessionLog()("pasteFromClipboard: mime data contains {} formats.", md->formats().size());
         for (int i = 0; i < md->formats().size(); ++i)
             sessionLog()("pasteFromClipboard[{}]: {}\n", i, md->formats().at(i).toStdString());
-        string const text = strip_if(normalize_crlf(clipboard->text(QClipboard::Clipboard)), strip);
-        if (text.empty())
+
+        auto const text = clipboard->text(QClipboard::Clipboard);
+
+        // 1 MB hard limit
+        if (text.size() > 1024 * 1024)
+        {
+            sessionLog()("Clipboard contains huge text. Ignoring.");
+            _display->post([this]() {
+                emit showNotification("Screenshot", QString::fromStdString("Paste is too big"));
+            });
+            return;
+        }
+        // 512 KB soft limit to ask user for permission
+        if (text.size() > 1024 * 512)
+        {
+            _pendingBigPaste = clipboard;
+            emit requestPermissionForPasteLargeFile();
+            sessionLog()("Clipboard contains huge text. Requesting permission.");
+            return;
+        }
+
+        string const strippedText = strip_if(normalize_crlf(clipboard->text(QClipboard::Clipboard)), strip);
+        sessionLog()("Size of text: {}", strippedText.size());
+        if (strippedText.empty())
             sessionLog()("Clipboard does not contain text.");
         else if (count == 1)
-            terminal().sendPaste(string_view { text });
+            terminal().sendPaste(string_view { strippedText });
         else
         {
             string fullPaste;
             for (unsigned i = 0; i < count; ++i)
-                fullPaste += text;
+                fullPaste += strippedText;
             terminal().sendPaste(string_view { fullPaste });
         }
     }
@@ -667,6 +692,26 @@ void TerminalSession::pasteFromClipboard(unsigned count, bool strip)
         sessionLog()("Could not access clipboard.");
 }
 
+void TerminalSession::applyPendingPaste(bool allow, bool remember)
+{
+    sessionLog()("applyPendingPaste: allow={}, remember={}", allow, remember);
+    if (remember)
+        _rememberedPermissions[GuardedRole::BigPaste] = allow;
+
+    if (!_pendingBigPaste)
+        return;
+
+    if (!allow)
+    {
+        _pendingBigPaste = std::nullopt;
+        return;
+    }
+
+    auto* clipboard = _pendingBigPaste.value();
+    auto text = clipboard->text(QClipboard::Clipboard);
+    terminal().sendPaste(string_view { text.toStdString() });
+}
+
 void TerminalSession::onSelectionCompleted()
 {
     switch (_config.onMouseSelection.value())
@@ -701,10 +746,10 @@ void TerminalSession::requestWindowResize(LineCount lines, ColumnCount columns)
     _display->post([this, lines, columns]() { _display->resizeWindow(lines, columns); });
 }
 
-void TerminalSession::adaptToWidgetSize()
+void TerminalSession::resizeTerminalToDisplaySize()
 {
     if (_display)
-        _display->post([this]() { _display->adaptToWidgetSize(); });
+        _display->post([this]() { _display->resizeTerminalToDisplaySize(); });
 }
 
 void TerminalSession::requestWindowResize(Width width, Height height)
@@ -776,7 +821,7 @@ void TerminalSession::sendKeyEvent(Key key, Modifiers modifiers, KeyboardEventTy
         return;
     }
 
-    if (_profile.mouseHideWhileTyping.value())
+    if (_profile.mouse.value().hideWhileTyping)
         _display->setMouseCursorShape(MouseCursorShape::Hidden);
 
     if (eventType != KeyboardEventType::Release)
@@ -799,17 +844,18 @@ void TerminalSession::sendCharEvent(
                modifiers,
                crispy::escape(unicode::convert_to<char>(value)));
 
-    assert(_display != nullptr);
-
-    if (_terminatedAndWaitingForKeyPress && eventType == KeyboardEventType::Press)
+    if (_display)
     {
-        sessionLog()("Terminated and waiting for key press. Closing display.");
-        _display->closeDisplay();
-        return;
-    }
+        if (_terminatedAndWaitingForKeyPress && eventType == KeyboardEventType::Press)
+        {
+            sessionLog()("Terminated and waiting for key press. Closing display.");
+            _display->closeDisplay();
+            return;
+        }
 
-    if (_profile.mouseHideWhileTyping.value())
-        _display->setMouseCursorShape(MouseCursorShape::Hidden);
+        if (_profile.mouse.value().hideWhileTyping)
+            _display->setMouseCursorShape(MouseCursorShape::Hidden);
+    }
 
     if (eventType != KeyboardEventType::Release)
     {
@@ -897,7 +943,7 @@ void TerminalSession::sendFocusInEvent()
     terminal().sendFocusInEvent();
 
     if (_display)
-        _display->setBlurBehind(_profile.backgroundBlur.value());
+        _display->setBlurBehind(_profile.background.value().blur);
 
     scheduleRedraw();
 }
@@ -1011,10 +1057,10 @@ bool TerminalSession::operator()(actions::DecreaseFontSize)
 
 bool TerminalSession::operator()(actions::DecreaseOpacity)
 {
-    if (static_cast<uint8_t>(_profile.backgroundOpacity.value()) == 0)
+    if (static_cast<uint8_t>(_profile.background.value().opacity) == 0)
         return true;
 
-    --_profile.backgroundOpacity.value();
+    --_profile.background.value().opacity;
 
     emit opacityChanged();
 
@@ -1072,9 +1118,9 @@ bool TerminalSession::operator()(actions::IncreaseFontSize)
 
 bool TerminalSession::operator()(actions::IncreaseOpacity)
 {
-    if (static_cast<uint8_t>(_profile.backgroundOpacity.value()) >= std::numeric_limits<uint8_t>::max())
+    if (static_cast<uint8_t>(_profile.background.value().opacity) >= std::numeric_limits<uint8_t>::max())
         return true;
-    ++_profile.backgroundOpacity.value();
+    ++_profile.background.value().opacity;
 
     emit opacityChanged();
 
@@ -1183,9 +1229,36 @@ bool TerminalSession::operator()(actions::ScreenshotVT)
     return true;
 }
 
+bool TerminalSession::operator()(actions::SaveScreenshot)
+{
+    auto savePath =
+        app().dumpStateAtExit().value_or(crispy::app::instance()->localStateDir())
+        / fs::path(std::format("contour-screenshot-{:%Y-%m-%d-%H-%M-%S}.png", chrono::system_clock::now()));
+
+    _display->setScreenshotOutput(savePath);
+    auto message = std::format("Saving screenshot to {}", savePath.string());
+    sessionLog()(message);
+
+    _display->post(
+        [this, message]() { emit showNotification("Screenshot", QString::fromStdString(message)); });
+    return true;
+}
+
+bool TerminalSession::operator()(actions::CopyScreenshot)
+{
+    _display->setScreenshotOutput(std::monostate {});
+    auto message = std::format("Saving screenshot to clipboard");
+    sessionLog()(message);
+
+    _display->post(
+        [this, message]() { emit showNotification("Screenshot", QString::fromStdString(message)); });
+
+    return true;
+}
+
 bool TerminalSession::operator()(actions::ScrollDown)
 {
-    terminal().viewport().scrollDown(_profile.historyScrollMultiplier.value());
+    terminal().viewport().scrollDown(_profile.history.value().historyScrollMultiplier);
     return true;
 }
 
@@ -1241,7 +1314,7 @@ bool TerminalSession::operator()(actions::ScrollToTop)
 
 bool TerminalSession::operator()(actions::ScrollUp)
 {
-    terminal().viewport().scrollUp(_profile.historyScrollMultiplier.value());
+    terminal().viewport().scrollUp(_profile.history.value().historyScrollMultiplier);
     return true;
 }
 
@@ -1354,6 +1427,43 @@ bool TerminalSession::operator()(actions::WriteScreen const& event)
     terminal().writeToScreen(event.chars);
     return true;
 }
+
+bool TerminalSession::operator()(actions::CreateNewTab)
+{
+    emit createNewTab();
+    return true;
+}
+
+bool TerminalSession::operator()(actions::CloseTab)
+{
+    emit closeTab();
+    return true;
+}
+
+bool TerminalSession::operator()(actions::SwitchToTab const& event)
+{
+    emit switchToTab(event.position);
+    return true;
+}
+
+bool TerminalSession::operator()(actions::SwitchToPreviousTab)
+{
+    emit switchToPreviousTab();
+    return true;
+}
+
+bool TerminalSession::operator()(actions::SwitchToTabLeft)
+{
+    emit switchToTabLeft();
+    return true;
+}
+
+bool TerminalSession::operator()(actions::SwitchToTabRight)
+{
+    emit switchToTabRight();
+    return true;
+}
+
 // }}}
 // {{{ implementation helpers
 void TerminalSession::setDefaultCursor()
@@ -1395,7 +1505,7 @@ int TerminalSession::executeAllActions(std::vector<actions::Action> const& actio
     }
 
     auto const containsToggleKeybind = [](std::vector<actions::Action> const& actions) {
-        return std::any_of(actions.begin(), actions.end(), [](actions::Action const& action) {
+        return std::ranges::any_of(actions, [](actions::Action const& action) {
             return holds_alternative<actions::ToggleAllKeyMaps>(action);
         });
     };
@@ -1463,7 +1573,6 @@ void TerminalSession::activateProfile(string const& newProfileName)
     _profileName = newProfileName;
     _profile = *newProfile;
     configureTerminal();
-    configureDisplay();
 }
 
 void TerminalSession::configureTerminal()
@@ -1479,12 +1588,13 @@ void TerminalSession::configureTerminal()
 
     sessionLog()("Setting terminal ID to {}.", _profile.terminalId.value());
     _terminal.setTerminalId(_profile.terminalId.value());
-    _terminal.setMaxSixelColorRegisters(_config.maxImageColorRegisters.value());
-    _terminal.setMaxImageSize(_config.maxImageSize.value());
-    _terminal.setMode(vtbackend::DECMode::NoSixelScrolling, !_config.sixelScrolling.value());
-    _terminal.setStatusDisplay(_profile.initialStatusDisplayType.value());
-    sessionLog()(
-        "maxImageSize={}, sixelScrolling={}", _config.maxImageSize.value(), _config.sixelScrolling.value());
+    _terminal.setMaxSixelColorRegisters(_config.images.value().maxImageColorRegisters);
+    _terminal.setMaxImageSize(_config.images.value().maxImageSize);
+    _terminal.setMode(vtbackend::DECMode::NoSixelScrolling, !_config.images.value().sixelScrolling);
+    _terminal.setStatusDisplay(_profile.statusLine.value().initialType);
+    sessionLog()("maxImageSize={}, sixelScrolling={}",
+                 _config.images.value().maxImageSize,
+                 _config.images.value().sixelScrolling);
 
     // XXX
     // if (!terminalView.renderer().renderTargetAvailable())
@@ -1492,10 +1602,11 @@ void TerminalSession::configureTerminal()
 
     configureCursor(_profile.modeInsert.value().cursor);
     updateColorPreference(_app.colorPreference());
-    _terminal.setMaxHistoryLineCount(_profile.maxHistoryLineCount.value());
+    _terminal.setMaxHistoryLineCount(_profile.history.value().maxHistoryLineCount);
     _terminal.setHighlightTimeout(_profile.highlightTimeout.value());
     _terminal.viewport().setScrollOff(_profile.modalCursorScrollOff.value());
     _terminal.inputHandler().setSearchModeSwitch(_profile.searchModeSwitch.value());
+    _terminal.settings().isInsertAfterYank = _profile.insertAfterYank.value();
 }
 
 void TerminalSession::configureCursor(config::CursorConfig const& cursorConfig)
@@ -1515,7 +1626,7 @@ void TerminalSession::configureDisplay()
         return;
 
     sessionLog()("Configuring display.");
-    _display->setBlurBehind(_profile.backgroundBlur.value());
+    _display->setBlurBehind(_profile.background.value().blur);
 
     {
         auto const dpr = _display->contentScale();
@@ -1535,10 +1646,10 @@ void TerminalSession::configureDisplay()
 
     _terminal.setRefreshRate(_display->refreshRate());
     _display->setFonts(_profile.fonts.value());
-    adaptToWidgetSize();
+    resizeTerminalToDisplaySize();
 
-    _display->setHyperlinkDecoration(_profile.hyperlinkDecorationNormal.value(),
-                                     _profile.hyperlinkDecorationHover.value());
+    _display->setHyperlinkDecoration(_profile.hyperlinkDecoration.value().normal,
+                                     _profile.hyperlinkDecoration.value().hover);
 
     setWindowTitle(_terminal.windowTitle());
 }
