@@ -3,10 +3,10 @@
 
 #include <crispy/assert.h>
 
-#include <fmt/format.h>
-
 #include <QtCore/QtGlobal>
 
+#include <format>
+
 #if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
     #include <QtGui/QSurfaceFormat>
 
@@ -23,7 +23,7 @@
 namespace contour::display
 {
 
-enum class ShaderClass
+enum class ShaderClass : uint8_t
 {
     Background,
     Text
