@@ -20,16 +20,16 @@
 #include <QtGui/QOpenGLExtraFunctions>
 #include <QtGui/QVector4D>
 #include <QtMultimedia/QMediaPlayer>
+#include <QtQml/QtQml>
 #include <QtQuick/QQuickItem>
 
-#include <QtQml>
-#include <atomic>
-#include <fstream>
+#include <filesystem>
 #include <memory>
 #include <optional>
-#include <vector>
-
-#include <qqml.h>
+#include <variant>
+#if defined(CONTOUR_PERF_STATS)
+    #include <atomic>
+#endif
 
 namespace contour::display
 {
@@ -84,7 +84,7 @@ class TerminalDisplay: public QQuickItem
 
     [[nodiscard]] bool hasSession() const noexcept { return _session != nullptr; }
 
-    // NB: Use TerminalSession.attachDisplay, that one is calling this here.
+    // NB: Use TerminalSession.attachDisplay, that one is calling this here. TODO(PR) ?
     void setSession(TerminalSession* newSession);
 
     [[nodiscard]] TerminalSession& session() noexcept
@@ -124,8 +124,7 @@ class TerminalDisplay: public QQuickItem
     [[nodiscard]] vtbackend::ImageSize pixelSize() const;
     [[nodiscard]] vtbackend::ImageSize cellSize() const;
 
-    // general events
-    void adaptToWidgetSize();
+    void resizeTerminalToDisplaySize();
 
     // (user requested) actions
     vtbackend::FontDef getFontDef();
@@ -152,6 +151,7 @@ class TerminalDisplay: public QQuickItem
     void onSelectionCompleted();
     void bufferChanged(vtbackend::ScreenType);
     void discardImage(vtbackend::Image const&);
+    void setScreenshotOutput(auto&& where) { _saveScreenshot = std::forward<decltype(where)>(where); }
     // }}}
 
     [[nodiscard]] std::optional<double> queryContentScaleOverride() const;
@@ -205,31 +205,35 @@ class TerminalDisplay: public QQuickItem
         return unbox(terminal().currentScreen().historyLineCount());
     }
 
-  private:
-    // helper methods
-    //
-    void doDumpStateInternal();
-    void createRenderer();
-    [[nodiscard]] QMatrix4x4 createModelMatrix() const;
-    void configureScreenHooks();
-    void watchKdeDpiSetting();
-    [[nodiscard]] float uptime() const noexcept;
-
     [[nodiscard]] vtbackend::PageSize calculatePageSize() const
     {
         assert(_renderer);
         assert(_session);
 
         // auto const availablePixels = gridMetrics().cellSize * _session->terminal().pageSize();
-        auto const availablePixels = vtbackend::ImageSize { vtbackend::Width::cast_from(width()),
-                                                            vtbackend::Height::cast_from(height()) };
-        return pageSizeForPixels(
-            availablePixels,
-            _renderer->gridMetrics().cellSize,
-            applyContentScale(_session->profile().margins.value(), _session->contentScale()));
+        auto const availablePixels = pixelSize();
+        return pageSizeForPixels(availablePixels,
+                                 _renderer->gridMetrics().cellSize,
+                                 applyContentScale(_session->profile().margins.value(), contentScale()));
     }
 
+  private:
+    // helper methods
+    //
+    void doDumpStateInternal();
+    QImage screenshot();
+    void createRenderer();
+    [[nodiscard]] QMatrix4x4 createModelMatrix() const;
+    void configureScreenHooks();
+    void watchKdeDpiSetting();
+    [[nodiscard]] float uptime() const noexcept;
+
     void updateMinimumSize();
+
+    // Updates the recommended size in (virtual pixels) based on:
+    // - the grid cell size (based on the current font size and DPI),
+    // - configured window margins, and
+    // - the terminal's screen size.
     void updateImplicitSize();
 
     void statsSummary();
@@ -266,12 +270,13 @@ class TerminalDisplay: public QQuickItem
     bool _renderingPressure = false;
     display::OpenGLRenderer* _renderTarget = nullptr;
     bool _maximizedState = false;
-
+    bool _sessionChanged = false;
     // update() timer used to animate the blinking cursor.
     QTimer _updateTimer;
 
     RenderStateManager _state;
     bool _doDumpState = false;
+    std::optional<std::variant<std::filesystem::path, std::monostate>> _saveScreenshot { std::nullopt };
 
     QFileSystemWatcher _filesystemWatcher;
     QMediaPlayer _mediaPlayer;
