@@ -207,6 +207,12 @@ class TraceHandler: public SequenceHandler
     PendingSequenceQueue _pendingSequences = {};
 };
 
+struct TabsInfo
+{
+    size_t tabCount = 1;
+    size_t activeTabPosition = 1;
+};
+
 /// Terminal API to manage input and output devices of a pseudo terminal, such as keyboard, mouse, and screen.
 ///
 /// With a terminal being attached to a Process, the terminal's screen
@@ -620,6 +626,10 @@ class Terminal
         return _viCommands.cursorPosition;
     }
     void moveNormalModeCursorTo(CellLocation pos) noexcept { _viCommands.moveCursorTo(pos); }
+    void addLineOffsetToJumpHistory(LineOffset offset) noexcept
+    {
+        _viCommands.addLineOffsetToJumpHistory(offset);
+    }
 
     // {{{ cursor management
     CursorDisplay cursorDisplay() const noexcept { return _settings.cursorDisplay; }
@@ -955,6 +965,9 @@ class Terminal
     void setStatusLineDefinition(StatusLineDefinition&& definition);
     void resetStatusLineDefinition();
 
+    TabsInfo guiTabsInfoForStatusLine() const noexcept { return _guiTabInfoForStatusLine; }
+    void setGuiTabInfoForStatusLine(TabsInfo info) { _guiTabInfoForStatusLine = info; }
+
   private:
     void mainLoop();
     void fillRenderBufferInternal(RenderBuffer& output, bool includeSelection);
@@ -1067,6 +1080,8 @@ class Terminal
     Viewport _viewport;
     StatusLineDefinition _indicatorStatusLineDefinition;
 
+    TabsInfo _guiTabInfoForStatusLine;
+
     // {{{ selection states
     std::unique_ptr<Selection> _selection;
     TheSelectionHelper _selectionHelper;
